@@ -30,7 +30,66 @@ class ETreeXMLSchemaTestCase(HelperTestCase):
 ''')
         schema = etree.XMLSchema(schema)
         self.assertTrue(schema.validate(tree_valid))
-        self.assertTrue(not schema.validate(tree_invalid))
+        self.assertFalse(schema.validate(tree_invalid))
+        self.assertTrue(schema.validate(tree_valid))     # retry valid
+        self.assertFalse(schema.validate(tree_invalid))  # retry invalid
+
+    def test_xmlschema_error_log(self):
+        tree_valid = self.parse('<a><b></b></a>')
+        tree_invalid = self.parse('<a><c></c></a>')
+        schema = self.parse('''
+<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+  <xsd:element name="a" type="AType"/>
+  <xsd:complexType name="AType">
+    <xsd:sequence>
+      <xsd:element name="b" type="xsd:string" />
+    </xsd:sequence>
+  </xsd:complexType>
+</xsd:schema>
+''')
+        schema = etree.XMLSchema(schema)
+        self.assertTrue(schema.validate(tree_valid))
+        self.assertFalse(schema.error_log.filter_from_errors())
+
+        self.assertFalse(schema.validate(tree_invalid))
+        self.assertTrue(schema.error_log.filter_from_errors())
+        self.assertTrue(schema.error_log.filter_types(
+            etree.ErrorTypes.SCHEMAV_ELEMENT_CONTENT))
+
+        self.assertTrue(schema.validate(tree_valid))
+        self.assertFalse(schema.error_log.filter_from_errors())
+
+        self.assertFalse(schema.validate(tree_invalid))
+        self.assertTrue(schema.error_log.filter_from_errors())
+        self.assertTrue(schema.error_log.filter_types(
+            etree.ErrorTypes.SCHEMAV_ELEMENT_CONTENT))
+
+    def test_xmlschema_error_log_path(self):
+        """We don't have a guarantee that there will always be a path
+        for a _LogEntry object (or even a node for which to determina
+        a path), but at least when this test was created schema validation
+        errors always got a node and an XPath value. If that ever changes,
+        we can modify this test to something like:
+            self.assertTrue(error_path is None or tree_path == error_path)
+        That way, we can at least verify that if we did get a path value
+        it wasn't bogus.
+        """
+        tree = self.parse('<a><b>42</b><b>dada</b></a>')
+        schema = self.parse('''
+<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+  <xsd:element name="a" type="AType"/>
+  <xsd:complexType name="AType">
+    <xsd:sequence>
+      <xsd:element name="b" type="xsd:integer" maxOccurs="2"/>
+    </xsd:sequence>
+  </xsd:complexType>
+</xsd:schema>
+''')
+        schema = etree.XMLSchema(schema)
+        schema.validate(tree)
+        tree_path = tree.getpath(tree.findall('b')[1])
+        error_path = schema.error_log[0].path
+        self.assertTrue(tree_path == error_path)
 
     def test_xmlschema_default_attributes(self):
         schema = self.parse('''
@@ -200,6 +259,23 @@ class ETreeXMLSchemaTestCase(HelperTestCase):
         self.assertEqual([('end', 'b'), ('end', 'a')],
                           events)
 
+    def test_xmlschema_iterparse_incomplete(self):
+        schema = self.parse('''
+<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
+  <xsd:element name="a" type="AType"/>
+  <xsd:complexType name="AType">
+    <xsd:sequence>
+      <xsd:element name="b" type="xsd:string" />
+    </xsd:sequence>
+  </xsd:complexType>
+</xsd:schema>
+''')
+        schema = etree.XMLSchema(schema)
+        xml = BytesIO('<a><b></b></a>')
+        event, element = next(iter(etree.iterparse(xml, schema=schema)))
+        self.assertEqual('end', event)
+        self.assertEqual('b', element.tag)
+
     def test_xmlschema_iterparse_fail(self):
         schema = self.parse('''
 <xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema">
@@ -290,7 +366,27 @@ class ETreeXMLSchemaTestCase(HelperTestCase):
 </xsd:schema>
 ''')
         self.assertTrue(tree_valid.xmlschema(schema))
-        self.assertTrue(not tree_invalid.xmlschema(schema))
+        self.assertFalse(tree_invalid.xmlschema(schema))
+
+    def test_create_from_partial_doc(self):
+        # this used to crash because the schema part was not properly copied out
+        wsdl = self.parse('''\
+<wsdl:definitions
+   xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
+   xmlns:xs="http://www.w3.org/2001/XMLSchema">
+ <wsdl:types>
+  <xs:schema>
+  </xs:schema>
+ </wsdl:types>
+</wsdl:definitions>
+        ''')
+        schema_element = wsdl.find(
+            "{http://schemas.xmlsoap.org/wsdl/}types/"
+            "{http://www.w3.org/2001/XMLSchema}schema"
+        )
+        etree.XMLSchema(schema_element)
+        etree.XMLSchema(schema_element)
+        etree.XMLSchema(schema_element)
 
 
 class ETreeXMLSchemaResolversTestCase(HelperTestCase):
